/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis;

import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Eine Lehrnmethode nach der ein Regelobjekt gesteigert werden kann.
 *
 * @author Bernhard Jung
 *
 */
public enum Lernmethode {

    KEINE( "Lernmethode.Keine" ),
    SELBSTSTUDIUM ( "Lernmethode.Selbststudium" ),
    GEGENSEITIG ( "Lernmethode.Gegenseitig" ),
    LEHRMEISTER( "Lernmethode.Lehrmeister" ),
    SPEZIELLE_ERFAHRUNG( "Lernmethode.SpezielleErfahrung" );

    /** Die volle Bezeichnung fuer die Lernmethode. */
    private final String name;

    /**
     * @param dieId der Lernmethode
     */
    private Lernmethode( final String dieId ) {

        name = I18n.i18n( dieId + ".Name" );
    }

    /**
     * Liefert den Steigerungsfaktor eines Regelobjekts, modifiziert nach dieser
     * Lernmethode.
     *
     * @param steigerbar
     *            Objekt dessen Steigerungsfaktor ermittelt werden soll.
     * @return den Steigerungsfaktor des Regelobjekts nach der Lernmethode.
     */
    public final int bestimmeSteigerungsfaktor(
            final Steigerbar steigerbar ) {

        switch ( this ) {
        case SPEZIELLE_ERFAHRUNG:
        case LEHRMEISTER:
            return ( -1 + steigerbar.getSteigerungsspalte() );

        case GEGENSEITIG:
        case KEINE:
            return steigerbar.getSteigerungsspalte();

        case SELBSTSTUDIUM:
            return ( 1 + steigerbar.getSteigerungsspalte() );

        default:
            throw new RuntimeException( String.format(
                    "Die Lernmethode %1$s ist noch nicht in die Methode " +
                    "eingebunden.", this.getName() ) );
        }
    }

    /** @return den Namen der Lernmethode, zB "Spezielle Erfahrung". */
    public final String getName() {
        return name;
    }

    /**
     * @return den Namen der Lernmethode.
     */
    @Override
    public String toString() {
        return name;
    }
}
