/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis;

import static de.bernhardjung.dsaprogramm.daten.xml.SchildXmlTags.*;

import java.text.Collator;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

import de.bernhardjung.dsaprogramm.i18n.I18n;

public class Schild implements Comparable< Schild > {

    /** Zum Sortieren der Schilde nach Namen. */
    private static final Collator COLLATOR = Collator.getInstance();

    /** Bruchfaktor */
    private int bf;

    private final String id;

    /** Intiativemodifikator */
    private int ini;

    /** Der Name des Schildes */
    private String name;

    /** Die Groesse des Schildes als Fernkampfdeckung. */
    private SchildTyp typ;

    /** Attackemodifikator */
    private int wmAt;

    /** Parademodifikator */
    private int wmPa;

    public Schild( final String dieId ) {

        id = dieId;

        if ( I18n.hatLokalisierungstext( id ) ) {

            name = I18n.i18n( id );

        } else {

            name = id;
        }

        bf = 3;

        ini = -1;

        typ = SchildTyp.GROSS;

        wmAt = -1;

        wmPa = 3;
    }

    /** @return den Bruchfaktor des Schildes. */
    public int getBf() {
        return bf;
    }

    /**
     * Aendert den Bruchfaktor.
     * @param bf Der neue Bruchfaktor.
     */
    public void setBf( final int bf ) {
        this.bf = bf;
    }

    /**
     * @return Der Initiativemodifikator.
     */
    public int getIni() {
        return ini;
    }

    /**
     * Aendert den Initiativemodifikator
     *
     * @param ini Der neue Initiativemodifikator.
     */
    public void setIni( final int ini ) {
        this.ini = ini;
    }

    /** @return die Groesse des Schildes als Fernkampfdeckung. */
    public SchildTyp getTyp() {
        return typ;
    }

    /** @param derTyp Die Groesse des Schildes als Fernkampfdeckung. */
    public void setTyp( final SchildTyp derTyp ) {
        typ = derTyp;
    }

    /** @return der Attackemodifikator. */
    public int getWmAt() {
        return wmAt;
    }

    /**
     * Aendert den Attackemodifikator.
     * @param wmAt Der neue Attackemodifikator.
     */
    public void setWmAt( final int wmAt ) {
        this.wmAt = wmAt;
    }

    /**
     * @return den Parademodifikator.
     */
    public int getWmPa() {
        return wmPa;
    }

    /**
     * @param wmPa der neue Parademodifikator
     */
    public void setWmPa( final int wmPa ) {
        this.wmPa = wmPa;
    }

    /**
     * @return die Id des Schildes.
     */
    public String getId() {
        return id;
    }

    /**
     * @return der Name des Schildes.
     */
    public String getName() {
        return name;
    }

    /**
     * @param name der neue Name des Schildes.
     */
    public void setName( final String name ) {
        this.name = name;
    }

    @Override
    public String toString() {
        return name;
    }

    /**
     * @return ein XML-Element, das den Schild repraesentiert.
     */
    public IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_SCHILD );

        element.setAttribute( ATTR_BF, String.valueOf( bf ) );

        element.setAttribute( ATTR_ID, id );

        element.setAttribute( ATTR_INI, String.valueOf( ini ) );

        if ( ( ! I18n.hatLokalisierungstext( id ) )
          || ( ! I18n.i18n( id ).equals( name ) ) ) {

            element.setAttribute( ATTR_NAME, name );
        }

        element.setAttribute( ATTR_TYP, typ.getId() );

        element.setAttribute( ATTR_WM_AT, String.valueOf( wmAt ) );

        element.setAttribute( ATTR_WM_PA, String.valueOf( wmPa ) );

        return element;
    }

    /** Sortiert die Schilde alphabetisch nach Namen. */
    public int compareTo( final Schild o ) {

        int ergebnis = COLLATOR.compare( getName(), o.getName() );

        if ( 0 == ergebnis ) {

            ergebnis = COLLATOR.compare( getId(), o.getId() );
        }

        return ergebnis;
    }
}
