/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.bernhardjung.dsaprogramm.i18n.I18n;

public class SchildTyp {

    private static final List< SchildTyp > TYPEN =
        new ArrayList< SchildTyp >();

    private static final Map< String, SchildTyp > TYPEN_MAP =
        new HashMap< String, SchildTyp >();

    public static final SchildTyp GROSS;

    static {
        erzeugeTyp( "SchildTyp.klein" );
        GROSS = erzeugeTyp( "SchildTyp.gross" );
        erzeugeTyp( "SchildTyp.sehrGross" );
    }

    private static final SchildTyp erzeugeTyp( final String dieId ) {

        final SchildTyp typ = new SchildTyp( dieId );

        TYPEN.add( typ );

        TYPEN_MAP.put( dieId, typ );

        return typ;
    }

    public static final SchildTyp getTyp( final String dieId ) {
        return TYPEN_MAP.get( dieId );
    }

    public static final List< SchildTyp > getTypen() {
        return new ArrayList< SchildTyp >( TYPEN );
    }

    private final String id;

    private final String name;

    private SchildTyp( final String dieId ) {

        id = dieId;

        name = I18n.i18n( id );
    }

    public String getId() {
        return id;
    }

    @Override
    public String toString() {
        return name;
    }
}
