/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis;

import java.util.Random;
import java.util.Vector;

import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Repraesentiert die im Spiel verwendeten Wuerfel.
 * Namentlich den 6-seitigen W6, den 20-seitigen W20
 * und den '3-seitigen' W3.
 *
 * @author Bernhard Jung
 */
public class Wuerfel {

    private static final Vector< Wuerfel > WUERFEL = new Vector< Wuerfel >();

    public static final Wuerfel W3 = new Wuerfel( 3 );
    public static final Wuerfel W6 = new Wuerfel( 6 );
    public static final Wuerfel W20 = new Wuerfel( 20 );

    static {

        WUERFEL.add( W3 );
        WUERFEL.add( W6 );
        WUERFEL.add( W20 );
    }

    /**
     * @return eine Liste von Wuerfeln.
     */
    public static final Vector< Wuerfel > getWuerfel() {
        return new Vector< Wuerfel >( WUERFEL );
    }

    /** Die Augenzahl des Wuerfels. */
    private final int augenzahl;

    /** Der Zufallsgenerator. */
    private final Random wuerfel;

    /** Die String-Repraesentation des Wuerfels. */
    private final String toString;

    /**
     * @param eineAugenzahl Die Augenzahl des Wuerfels.
     */
    public Wuerfel( final int eineAugenzahl ) {
        augenzahl = eineAugenzahl;
        wuerfel = new Random();
        toString = I18n.i18n( "Wuerfel" ) + augenzahl;
    }

    /**
     * @return die Augenzahl des Wuerfels.
     */
    public int getAugenzahl() {
        return augenzahl;
    }

    /** @return das zufaellige Ergebnis eines Wuerfelwurfs. */
    public int wuerfeln() {
        return ( 1 + wuerfel.nextInt( augenzahl ) );
    }

    @Override
    public String toString() {
        return toString;
    }

}
