/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis.varianten;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;

/**
 * Liefert die Talente welche die Varianten fuer Begabung fuer Talent bilden.
 *
 * @author Bernhard Jung
 */
class VariantenBegabungTalent implements Varianten {

    /** Die Talente. */
    private final List< Talent > talente;

    VariantenBegabungTalent() {
        // Sicher gehen, dass die Talent auch initialisiert sind.
        final Talent talent = Talent.ABRICHTEN;
        talent.getName();

        talente = new ArrayList< Talent >();

        talente.addAll( Talentgruppe.GESELLSCHAFT );
        talente.addAll( Talentgruppe.HANDWERK );
        talente.addAll( Talentgruppe.KAMPF );
        talente.addAll( Talentgruppe.KOERPER );
        talente.addAll( Talentgruppe.NATUR );
        talente.addAll( Talentgruppe.WISSEN );

        Collections.sort( talente );
    }

    /**
     * @throws UnsupportedOperationException
     *             wenn die Methode aufgerufen wird.
     */
    public void einfuegen( final String variante ) {
        throw new UnsupportedOperationException(
                "Bei der VarianteBegabungTalent kann keine neue Variante " +
                "eingefuegt werden.");
    }

    /**
     * @param held wird nicht benoetigt und darf {@code null} sein.
     * @return eine Liste von Talenten.
     */
    public List<Object> getVarianten(final Held held) {
        return new ArrayList< Object >( talente );
    }

    /** @return {@code true} */
    public boolean hatVarianten() {
        return true;
    }

    /** @return {@code false} */
    public boolean istFreieVariante() {
        return false;
    }

    /** Eine Aufzaehlung aller Varianten. */
    @Override
    public final String toString() {
        return getVarianten( null ).toString();
    }
}
