/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.basis.varianten;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;

/**
 * Die Varianten (Talente) fuer den Vorteil Meisterhandwerk.
 *
 * @author Bernhard Jung
 *
 */
class VariantenMeisterhandwerk implements Varianten {

    /** Eine Liste von moeglichen Talenten. */
    private final List< Talent > varianten;

    /** Erzeugt eine Liste mit moglichen Talenten. */
    VariantenMeisterhandwerk() {

        varianten = new ArrayList< Talent >();

        varianten.add( Talent.MECHANIK );
        varianten.add( Talent.PFLANZENKUNDE );
        varianten.add( Talent.SAGEN_LEGENDEN );

        varianten.addAll( Talentgruppe.KOERPER );
        varianten.addAll( Talentgruppe.NATUR );
        varianten.addAll( Talentgruppe.GESELLSCHAFT );
        varianten.addAll( Talentgruppe.HANDWERK );

        Collections.sort( varianten );
    }

    /** @throws UnsupportedOperationException */
    public void einfuegen( final String variante ) {
        throw new UnsupportedOperationException();
    }

    /**
     * @param held wird nicht benoetigt und darf {@code null} sein.
     * @return eine Liste von moeglichen Talenten. */
    public List< Object > getVarianten(final Held held) {
        return new ArrayList< Object >( varianten );
    }

    /** @return {@code true} */
    public boolean hatVarianten() {
        return true;
    }

    /** @return {@code false} */
    public boolean istFreieVariante() {
        return false;
    }

    /** Eine Aufzaehlung aller Varianten. */
    @Override
    public final String toString() {
        return "Meisterhandwerk: " + getVarianten( null ).toString();
    }
}
