/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine.bonus;

import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;

/**
 * Ein Interface fuer alle Arten von Talentboni, die von Rasse, Kultur
 * oder Profession vergeben werden.
 *
 * @author Bernhard Jung
 */
public interface Talentbonus extends Comparable< Talentbonus >{

    /**
     * @return die GP-Kosten, die einem Baustein, durch den Bonus entstehen.
     */
    public double getGpKosten();

    /**
     * Liefert ein Talent von den Talenten des Bonus.
     * Es handelt sich dabei um ein beliebiges Talent.
     * Diese Methode ist nur zum Sortieren der Boni gedacht und sollte zu
     * keinem anderen Zweck aufgerufen werden.
     *
     * @return ein Talent.
     */
    public Talent getTalent();

    /**
     * @return die Talentgruppe zu welcher der Bonus gehoert.
     */
    public Talentgruppe getTalentgruppe();

    /** @return den Typ des Talent-Bonus. */
    public TalentbonusTyp getTyp();

    /** @return die XML-Repraesentation des Bonus. */
    public IXMLElement toXml();

}
