/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.bausteine.bonus.talentlisten;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import net.n3.nanoxml.XMLElement;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.talente.Talent;
import de.bernhardjung.dsaprogramm.talente.Talentgruppe;

/**
 * Eine Liste die fuer einen Helden die Schriften seiner Muttersprache
 * enthaelt.
 * Fuer die Profession Magier.
 *
 * @author Bernhard Jung
 */
class TalentListeSchriftMuttersprache implements TalentListe {

    private static final String BESCHREIBUNG =
        I18n.i18n( "TalentListeSchriftMuttersprache" );

    /** @return 'Die Schrift der Muttersprache' */
    public final String getBeschreibung() {
        return BESCHREIBUNG;
    }

    /** @return Kusliker Zeichen */
    public final List< Talent > getTalente() {
        return Arrays.asList( (Talent) Talent.KUSLIKER_ZEICHEN );
    }

    /**
     * @return die Schriften der Muttersprache des Helden.
     */
    public final List< Talent > getTalente( final Held held ) {
        return new ArrayList< Talent >( held.getMuttersprache().getSchriften());
    }

    /** @return {@link Talentgruppe#SCHRIFTEN} */
    public final Talentgruppe getTalentgruppe() {
        return Talentgruppe.SCHRIFTEN;
    }

    /** @return die Beschreibung der Liste. */
    @Override
    public final String toString() {
        return BESCHREIBUNG;
    }

    /** {@inheritDoc} */
    public final IXMLElement toXml() {

        final IXMLElement element = new XMLElement( TAG_TALENTLISTE );

        final IXMLElement mutterspracheElement =
            new XMLElement( TAG_SCHRIFTEN_MUTTERSPRACHE );

        element.addChild( mutterspracheElement );

        return element;
    }
}
