/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.daten.factories.bausteine;

import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_BONUS;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_HAUSZAUBER;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_VARIANTE;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.ATTR_ZAUBER;
import static de.bernhardjung.dsaprogramm.daten.xml.HeldenbausteinXmlTags.TAG_ZAUBER_BONI;

import java.util.ArrayList;
import java.util.List;

import net.n3.nanoxml.IXMLElement;
import de.bernhardjung.dsaprogramm.bausteine.bonus.Zauberbonus;
import de.bernhardjung.dsaprogramm.zauber.Zauber;

/**
 * Erzeugt die Boni auf die Zauber aus dem XML-Dokument.
 * @author Bernhard Jung
 *
 */
public class FactoryZauberBoniIn {

    /**
     * Erzeugt die Zauberboni aus der XML-Datei.
     *
     * @param wurzelElement des XML-Datei
     * @return die Liste der Zauberboni oder eine leere Liste, wenn es keine
     *         Boni auf Zauber gibt.
     */
    public final List< Zauberbonus > erzeugen( final IXMLElement wurzelElement ) {

        final List< Zauberbonus > boni = new ArrayList< Zauberbonus >();

        final IXMLElement boniElement =
            wurzelElement.getFirstChildNamed( TAG_ZAUBER_BONI );

        if ( null != boniElement ) {

            for ( final IXMLElement bonus : boniElement.getChildren() ) {

                final String zauberId = bonus.getAttribute( ATTR_ZAUBER );

                final Zauber zauber = Zauber.getZauber( zauberId );

                final String variantenId = bonus.getAttribute( ATTR_VARIANTE );

                final int stufenbonus =
                    Integer.valueOf( bonus.getAttribute( ATTR_BONUS ) );

                final boolean hauszauber =
                    Boolean.valueOf( bonus.getAttribute( ATTR_HAUSZAUBER ) );

                if ( null != variantenId ) {

                    boni.add( new Zauberbonus( zauber,
                                               variantenId,
                                               stufenbonus,
                                               hauszauber ) );

                } else {

                    boni.add( new Zauberbonus( zauber,
                                           stufenbonus,
                                           hauszauber ) );
                }
            }
        }

        return boni;
    }
}
