/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.URI;
import java.net.URL;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.factories.ButtonBarFactory;
import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.Start;
import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Der About-Dialog mit den Infos ueber das Programm.
 *
 * @author Bernhard Jung
 *
 */
public class DialogInfo extends JDialog {

    /** Der Link zur Hompage des Programms. */
    private static final String LINK = I18n.i18n( "DialogInfo.Link" );

    /** Meine E-Mail Adresse fuer das Programm. */
    private static final String MAIL = I18n.i18n( "DialogInfo.Mail" );

    public DialogInfo( final JFrame hauptfenster ) {
        super( hauptfenster, I18n.i18n( "DialogInfo.Titel" ) );

        setLayout( new FormLayout( "7dlu, F:P:G, 7dlu",
                                   "14dlu, P, 1dlu, F:300px:G, 7dlu, P, 7dlu" ) );

        final JLabel ueberschrift =
            new JLabel( Util.ladeIcon( I18n.i18n( "DialogInfo.Ueberschrift")));

        final JTabbedPane tabbedPane = erzeugeTabbedPane();

        final JPanel schliessenPanel = erzeugePanelSchliessen();

        final CellConstraints cc = new CellConstraints();

        add( ueberschrift, cc.xy( 2, 2 ) );

        add( tabbedPane, cc.xy( 2, 4 ) );

        add( schliessenPanel, cc.xy( 2, 6 ) );

        pack();

        setLocationRelativeTo( hauptfenster );
    }

/*
    /**
     * Erzeugt einen Label, der aussieht wie ein Hypertext-Link im Browser,
     * also blau und unterstrichen.
     *
     * @param text
     *            fuer das Label
     * @return ein Label.
     * /
    private JLabel erzeugeLinkLabel( final String text ) {

        final JLabel link = new JLabel( "<html><u>" + text + "</u>");

        link.setForeground( Color.BLUE );

        link.setCursor( Cursor.getPredefinedCursor( Cursor.HAND_CURSOR ) );

        return link;
    }
*/
    /**
     * Erzeugt einen JButton, mit dem der Browser gestaret werden kann, oder
     * ein JLabel da einfach nur eine URL darstellt. Je nachdem ob die
     * Operation {@link Desktop#browse(URI)} unterstuetzt wird.
     *
     * @return ein Label oder ein Button.
     */
    private JComponent erzeugeLink() {

        final JComponent link;
/*
        if ( Desktop.isDesktopSupported()
                && Desktop.getDesktop().isSupported( Action.BROWSE ) ) {

            link = erzeugeLinkLabel( LINK );

            link.addMouseListener( new MouseAdapter() {
                public final void mouseClicked( final MouseEvent e ) {
                    try {
                        Desktop.getDesktop().browse( new URI( "http://" + LINK ) );
                    } catch ( final Exception e1 ) {
                        e1.printStackTrace();
                    }
                }
            } );

        } else {
*/
            link = new JLabel( LINK );

        //}

        return link;
    }

    /**
     * Je nachdem ob {@link Desktop#mail(URI)} unterstuetzt wird erzeugt die
     * Methode einen Knopf der den Mail-Client oeffnet oder ein Label, das eine
     * E-Mail Adresse anzeigt.
     *
     * @return ein Button oder ein Label.
     */
    private final JComponent erzeugeMail() {

        final JComponent mail;

        /*
        if ( Desktop.isDesktopSupported()
                && Desktop.getDesktop().isSupported( Action.MAIL ) ) {

            mail = erzeugeLinkLabel( MAIL );

            mail.addMouseListener( new MouseAdapter() {
                public final void mouseClicked( final MouseEvent e ) {
                    try {
                        Desktop.getDesktop().mail(new URI( "mailto:" + MAIL ));
                    } catch ( final Exception e1 ) {
                        e1.printStackTrace();
                    }
                }
            } );

        } else {
        */
            mail = new JLabel( MAIL );
        //}

        return mail;
    }

    private final JScrollPane erzeugeTextPane( final String file )
    {
        final JEditorPane textPane = new JEditorPane();
        final URL textUrl = Start.class.getResource( file );
        try {
            textPane.setPage( textUrl );
        } catch (final IOException e ) {
            e.printStackTrace();
        }
        textPane.setEditable( false );

        final JScrollPane scrollPane = new JScrollPane( textPane );
        
        scrollPane.setPreferredSize( new Dimension( 100, 100 ) );
        
        return scrollPane;
    }
    
    /**
     * @return Das Register mit Copyright Vermerk, Lizenztexten, Disclaimer...
     */
    private final JTabbedPane erzeugeTabbedPane() {

        final JTabbedPane tabbedPane = new JTabbedPane();

        tabbedPane.addTab( I18n.i18n( "DialogInfo.Ueber" ),
                           erzeugePanelUeber() );

        tabbedPane.addTab( I18n.i18n( "DialogInfo.Lizenz"),
                           erzeugeTabLizenzen() );

        tabbedPane.addTab( I18n.i18n( "DialogInfo.Disclaimer"),
                           erzeugeTextPane( "ulisses.html" ) );

        return tabbedPane;
    }

    private final JTabbedPane erzeugeTabLizenzen()
    {
        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );
        
        tabbedPane.addTab( I18n.i18n( "DialogInfo.GPL" ), 
                erzeugeTextPane( "gpl.html" ) );
        
        tabbedPane.addTab( I18n.i18n( "DialogInfo.jgoodies" ), 
                erzeugeTextPane( "jgoodies.html" ) );
        
        return tabbedPane;
    }
    
    /**
     * @return ein Copyright-Vermerk fuer das Programm.
     */
    private final JPanel erzeugePanelUeber() {
        final JLabel charaktergenerator;
        charaktergenerator = new JLabel( I18n.i18n( "Programm.Name" ) );

        final JLabel copyright = new JLabel( "\u00A9 2007, Bernhard Jung");

        final JPanel panel;
        panel = new JPanel(
                new FormLayout( "C:P:G", "B:P:G, 7dlu, P, 7dlu, P, 7dlu, T:P:G" ) );

        final CellConstraints cc = new CellConstraints();

        panel.add( charaktergenerator, cc.xy( 1, 1 ) );

        panel.add( copyright,          cc.xy( 1, 3 ) );

        panel.add( erzeugeLink(),      cc.xy( 1, 5 ) );

        panel.add( erzeugeMail(),      cc.xy( 1, 7 ) );

        return panel;
    }

    /**
     * @return ein Panel mit einem Schliessen-Knopf.
     */
    private final JPanel erzeugePanelSchliessen() {

        final JButton bSchliessen;

        bSchliessen = new JButton( I18n.i18n( "Knopf.Schliessen" ) );
        bSchliessen.setMnemonic(
                Util.getKeyCode( I18n.i18n( "Knopf.Schliessen.Mnemonic" ) ) );

        bSchliessen.addActionListener( new ActionListener() {
            public final void actionPerformed( final ActionEvent e ) {
                setVisible( false );
            }
        } );

        return ButtonBarFactory.buildCloseBar( bSchliessen);
    }
}
