/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui;

import static de.bernhardjung.dsaprogramm.i18n.I18n.i18n;

import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Observer;

import javax.swing.JLabel;
import javax.swing.JPanel;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basiswerte.Basiswert;

/**
 * Ein Panel das die Basiswerte anzeigt.
 *
 * @author Bernhard Jung
 */
class PanelBasiswerte extends JPanel implements Observer {

    /**
     * Die Label welche die Basisstufen des Basiswerts anzeigen.
     * Die Label in der Liste sind den Basiswerten ueber ihren Index
     * zugeordnet d.h. an Position i steht das Label fuer den Basiswert mit
     * index i.
     */
    private final List< JLabel > labelBasisstufe;

    /**
     * Die Label welche die Stufen der Basiswerte anzeigen.
     * Die Label in der Liste sind den Basiswerten ueber ihren Index
     * zugeordnet d.h. an Position i steht das Label fuer den Basiswert mit
     * index i.
     */
    private final List< JLabel > labelGesamtstufe;

    /**
     * Die Label welche die Modifikationen der Basiswerte anzeigen.
     * Die Label in der Liste sind den Basiswerten ueber ihren Index
     * zugeordnet d.h. an Position i steht das Label fuer den Basiswert mit
     * index i.
     */
    private final List< JLabel > labelModifikation;

    PanelBasiswerte() {

        labelBasisstufe = new ArrayList< JLabel >();

        labelGesamtstufe = new ArrayList< JLabel >();

        labelModifikation = new ArrayList< JLabel >();

        for ( int i = Basiswert.getBasiswerte().length ; i > 0 ; --i ) {
            labelBasisstufe.add( Util.erzeugeLabelRechtsbuendig() );
            labelGesamtstufe.add( Util.erzeugeLabelRechtsbuendig() );
            labelModifikation.add( Util.erzeugeLabelRechtsbuendig() );
        }

        final FormLayout layout = new FormLayout(
                // Basiswert      Mod        +       Basis       =       Gesamt
                "7dlu, P:G, 4dlu, P:G, 4dlu, P, 4dlu, P:G, 4dlu, P, 4dlu, P:G, 7dlu",
                "7dlu, P,   4dlu, P:G, 4dlu, P:G, 4dlu, P:G, 4dlu, P:G, " +
                "4dlu, P:G, 4dlu, P:G, 4dlu, P:G, 4dlu, P:G, 4dlu, P:G, 7dlu" );

        setLayout( layout );

        final CellConstraints cc = new CellConstraints();

        add( Util.erzeugeUeberschrift( i18n( "PanelBasiswerte.Basiswert" ) ),
             cc.xy(  2, 2 ) );
        add( Util.erzeugeUeberschrift( i18n( "PanelBasiswerte.Mod" ) ),
             cc.xy(  4, 2 ) );
        add( Util.erzeugeUeberschrift( i18n( "PanelBasiswerte.Startwert" ) ),
             cc.xy(  8, 2 ) );
        add( Util.erzeugeUeberschrift( i18n( "PanelBasiswerte.Aktuell" ) ),
             cc.xy( 12, 2 ) );

        final Basiswert[] basiswerte = Basiswert.getBasiswerte();
        for ( int i = 0 ; i < basiswerte.length ; ++i ) {

            final int y = 2 * i + 4;
            int x = 0;

            add( new JLabel( basiswerte[ i ].getName() ), cc.xy( x+=2, y ) );
            add( labelModifikation.get( i ),              cc.xy( x+=2, y ) );
            add( new JLabel( "+" ),                       cc.xy( x+=2, y ) );
            add( labelBasisstufe.get( i ),                cc.xy( x+=2, y ) );
            add( new JLabel( "=" ),                       cc.xy( x+=2, y ) );
            add( labelGesamtstufe.get( i ),               cc.xy( x+=2, y ) );
        }
    }

    /**
     * Aendert den Held dessen Werte im Panel dargestellt werden.
     * @param held
     */
    final void setHeld( final Held held ) {
        held.addObserver( this );

        update( held, null );
    }

    /**
     * Aktualisiert die angezeigten Werte.
     *
     * @param o der Held der ueberwacht wird.
     * @param arg ungenutzt
     */
    public void update( final Observable o, final Object arg ) {

        final Held held = (Held) o;

        for ( final Basiswert basiswert : Basiswert.getBasiswerte() ) {

            final int index = basiswert.getIndex();

            String text = String.valueOf( basiswert.getBasisstufe( held ) );
            labelBasisstufe.get( index ).setText( text );

            text = String.valueOf( held.getStufe( basiswert ) );
            labelGesamtstufe.get( index ).setText( text );

            text = String.valueOf( basiswert.getModifikation( held ) );
            labelModifikation.get( index ).setText( text );
        }
    }

}
