/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.action;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;

import de.bernhardjung.dsaprogramm.gui.Hauptpanel;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Die Action zum beenden des Programms.
 *
 * @author Bernhard Jung
 */
public class ActionBeenden extends AbstractAction {

    /** Die Frage an den Benutzer ob er das Programm wirklich beenden will. */
    private static final String FRAGE = I18n.i18n( "ActionBeenden.Frage" );

    /** Die Taste mit der die Aktion im Menue direkt aktiviert werden kann. */
    private static final String MNEMONIC =
        I18n.i18n( "ActionBeenden.Mnemonic" );

    /** Das Tastenkuerzel mit dem die Aktion ausgeloest werden kann. */
    private static final String TASTEN = I18n.i18n( "ActionBeenden.Tasten" );

    /**
     * Titel fuer den Dialog der den Benutzer fragt ob der das Programm
     * beenden will.
     */
    private static final String TITEL = I18n.i18n( "ActionBeenden.Titel" );

    /** Das Hauptfenster der Anwendung. */
    private final Hauptpanel hauptpanel;

    public ActionBeenden( final Hauptpanel dasHauptpanel ) {

        super( I18n.i18n( "ActionBeenden.Name" ),
                Util.ladeIcon( "beenden" ) );

        hauptpanel = dasHauptpanel;

        putValue( MNEMONIC_KEY, Util.getKeyCode( MNEMONIC ) );
        putValue( ACCELERATOR_KEY, KeyStroke.getKeyStroke( TASTEN ) );
    }

    /**
     * Fragt den Benutzer ob er das Programm wirklich beenden will.
     *
     * @param e ungenutzt.
     */
    public void actionPerformed( final ActionEvent e ) {

        final int antwort = JOptionPane.showConfirmDialog(
                hauptpanel,
                FRAGE,
                TITEL,
                JOptionPane.YES_NO_OPTION );

        if ( JOptionPane.YES_OPTION == antwort ) {

            System.exit( 0 );

        }
    }

}
