/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.action;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.JFrame;
import javax.swing.KeyStroke;

import de.bernhardjung.dsaprogramm.gui.DialogInfo;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Die Aktion mit der der Info-Dialog angezeigt wird.
 *
 * @author Bernhard Jung
 */
public class ActionInfo extends AbstractAction {

    /** Das Tastenkurzel mit dem die Aktion ausgeloest werden kann. */
    private static final String TASTEN = I18n.i18n( "ActionInfo.Tasten" );

    /**
     * Die Taste mit der die Aktion im Menue direkt angesprungen werden kann.
     */
    private static final String MNEMONIC = I18n.i18n( "ActionInfo.Mnemonic" );

    /** Das Hauptfenster der Anwendung. */
    private final JFrame hauptfenster;

    /** Der Dialog den die Action anzeigt. */
    private DialogInfo dialogInfo;

    public ActionInfo( final JFrame dasHauptfenster ) {

        super( I18n.i18n( "ActionInfo.Name" ),
                Util.ladeIcon( "info" ) );

        hauptfenster = dasHauptfenster;

        putValue( MNEMONIC_KEY, Util.getKeyCode( MNEMONIC ) );
        putValue( ACCELERATOR_KEY, KeyStroke.getKeyStroke( TASTEN ) );
    }

    /**
     * Zeigt den Info-Dialog an.
     *
     * @param e ungenutzt.
     */
    public void actionPerformed( final ActionEvent e ) {

        if ( dialogInfo == null) {
             dialogInfo = new DialogInfo( hauptfenster );
        }

        dialogInfo.setVisible( ! dialogInfo.isVisible() );
    }

}
