/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.action;

import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.JFileChooser;

import de.bernhardjung.dsaprogramm.gui.DialogFehlermeldung;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.i18n.I18n;
import de.bernhardjung.dsaprogramm.util.Einstellungen;
import de.bernhardjung.dsaprogramm.util.FileFilter;

public abstract class ActionLadenHeldenbaustein extends AbstractAction {

    private final JFileChooser fileChooser;

    private final JComponent parent;

    private final String verzeichnis;

    public ActionLadenHeldenbaustein(
            final String nameId,
            final String mnemonicId,
            final String dateierweiterung,
            final String dasVerzeichnis,
            final JComponent parent ) {

        super( I18n.i18n( nameId ) );

        final String mnemonic = I18n.i18n( mnemonicId );
        putValue( MNEMONIC_KEY, Util.getKeyCode( mnemonic ) );

        fileChooser = new JFileChooser();
        fileChooser.setAcceptAllFileFilterUsed( false );
        fileChooser.setFileFilter( new FileFilter( dateierweiterung ) );

        this.parent = parent;

        verzeichnis = dasVerzeichnis;
    }

    public void actionPerformed( final ActionEvent e ) {

        final Einstellungen einstellungen = Einstellungen.getInstanz();

        if ( einstellungen.istDatenquelleVerzeichnis() ) {

            final String datenquelle = einstellungen.getDatenquelle();

            fileChooser.setCurrentDirectory(
                    new File( datenquelle, verzeichnis ) );
        }

        final int ergebnis = fileChooser.showOpenDialog( parent );

        if ( JFileChooser.APPROVE_OPTION == ergebnis ) {

            final File datei = fileChooser.getSelectedFile();

            if ( datei.exists() ) {

                try {

                    laden( datei );

                }  catch ( final Exception ex ) {

                    DialogFehlermeldung.zeigeDialog(
                            "Fehler beim Laden von " + datei.getName(), ex );
                }

            }

        }

    }

    abstract void laden( final File datei ) throws Exception;

}
