/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.gui.meta;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.jgoodies.forms.builder.DefaultFormBuilder;
import com.jgoodies.forms.layout.FormLayout;

import de.bernhardjung.dsaprogramm.bausteine.Heldenbaustein;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VerbilligteSonderfertigkeit;
import de.bernhardjung.dsaprogramm.bausteine.bonus.VerbilligteSonderfertigkeitStandard;
import de.bernhardjung.dsaprogramm.gui.Util;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeitgruppe;

class PanelMetaVerbilligteSonderfertigkeiten
extends JPanel {

    private final JCheckBox gelaendekunde;

    private final List< PanelMetaListe< Sonderfertigkeit > > panel;

    PanelMetaVerbilligteSonderfertigkeiten() {

        gelaendekunde = new JCheckBox(
                VerbilligteSonderfertigkeit.GELAENDEKUNDE.toString() );

        panel = new ArrayList< PanelMetaListe<Sonderfertigkeit> >();

        final JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.LEFT );

        for ( final Sonderfertigkeitgruppe gruppe:
                        Sonderfertigkeitgruppe.getGruppen() ) {

            final PanelMetaListe< Sonderfertigkeit > p =
                new PanelMetaListe< Sonderfertigkeit >( gruppe );

            panel.add( p );

            tabbedPane.addTab( gruppe.getKuerzel(),
                               Util.erzeugeScrollPane( p ) );
        }

        final DefaultFormBuilder builder =
            new DefaultFormBuilder( new FormLayout( "P" ) );

        builder.setDefaultDialogBorder();

        builder.append( gelaendekunde );

        setLayout( new BorderLayout() );

        add( builder.getPanel(), BorderLayout.PAGE_START );

        add( tabbedPane, BorderLayout.CENTER );
    }

    public final void anzeigen( final Heldenbaustein baustein ) {

        gelaendekunde.setSelected( false );

        final List< VerbilligteSonderfertigkeit > verbilligte =
            baustein.getVerbilligteSonderfertigkeiten();

        final List< Sonderfertigkeit > sonderfertigkeiten =
            new ArrayList< Sonderfertigkeit >( verbilligte.size() );

        for ( final VerbilligteSonderfertigkeit v : verbilligte ) {

            if ( v.hatAuswahl() ) {

                gelaendekunde.setSelected( true );

            } else {

                sonderfertigkeiten.add( v.getSonderfertigkeit() );
            }
        }

        for ( final PanelMetaListe< Sonderfertigkeit > p : panel ) {

            p.setGewaehlteElemente( sonderfertigkeiten );
        }
    }

    public final
    List< VerbilligteSonderfertigkeit > getVerbilligteSonderfertigkeiten() {

        final List< VerbilligteSonderfertigkeit > verbilligte =
            new ArrayList< VerbilligteSonderfertigkeit >();

        for ( final PanelMetaListe< Sonderfertigkeit > p : panel ) {

            for ( final Sonderfertigkeit sonderfertigkeit :
                                        p.getGewaehlteElemente() ) {

                verbilligte.add( new VerbilligteSonderfertigkeitStandard(
                        sonderfertigkeit ) );
            }
        }

        if ( gelaendekunde.isSelected() ) {

            verbilligte.add( VerbilligteSonderfertigkeit.GELAENDEKUNDE );
        }

        return verbilligte;
    }

    /** Waehlt alle Eintraege im Panel ab. */
    public void reset() {

        for ( final PanelMetaListe< Sonderfertigkeit > p : panel ) {

            p.reset();
        }

        gelaendekunde.setSelected( false );
    }
}
