/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.sonderfertigkeiten.kosten;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.basis.Lernmethode;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;

/**
 * Implementiert die Methode {@link #getGpKosten(Held, SonderfertigkeitWert)}
 * fuer Sonderfertigkeiten deren GP-Kosten 1/50 der AP-Kosten entsprechen.
 *
 * @author Bernhard Jung
 */
abstract class KostenAbstrakt implements Kosten {

    /**
     * Halbiert die Kosten bei Lernmethode Spezielle Erfahrung.
     */
    public int getApKosten(
            final Held held,
            final SonderfertigkeitWert wert,
            final Lernmethode lernmethode ) {

        int apKosten = getApKosten( held, wert );

        if ( Lernmethode.SPEZIELLE_ERFAHRUNG.equals( lernmethode ) ) {

            apKosten = Math.round( apKosten / 2F );
        }

        return apKosten;
    }

    /**
     * @return {@code Math.round( getApKosten() / 50F )}
     */
    public final int getGpKosten(final SonderfertigkeitWert wert) {
        return Math.round( getApKosten(wert) / 50F );
    }

    /**
     * @return {@code Math.round( getApKosten( held, wert ) / 50F )}
     */
    public final int getGpKosten( final Held held, final SonderfertigkeitWert wert ) {
        return Math.round( getApKosten( held, wert ) / 50F );
    }

    /**
     * @return {@code true}
     */
    public boolean hatGpKosten() {
        return true;
    }
}
