/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.sonderfertigkeiten.kosten;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

/**
 * Bestimmt die Kosten fuer Sonderfertigkeiten die einem festen Betrag an AP
 * kosten.
 */
class KostenFest extends KostenAbstrakt {

    /** Die AP-Kosten die fuer die Sonderfertigkeit zu zahlen sind. */
    private final int apKosten;

    KostenFest( final int dieApKosten ) {
        apKosten = dieApKosten;
    }

    /** @return Elfische Weltsicht. */
    public Collection<Object> getAbhaengigkeiten() {
        final List< Object > abhaengigkeiten = new ArrayList< Object >();

        abhaengigkeiten.add( VorNachteil.ELFISCHE_WELTSICHT );

        return abhaengigkeiten;
    }

    public int getApKosten(Sonderfertigkeit sonderfertigkeit) {
        return apKosten;
    }

    public int getApKosten(final SonderfertigkeitWert wert) {
        return apKosten;
    }

    public int getApKosten(
            final Held held,
            final SonderfertigkeitWert wert) {

        if ( held.hatVorgegeben( wert ) ) {
            return 0;
        }

        final float faktor = getFaktor( held, wert );

        return Math.round( faktor * apKosten );
    }

    protected float getFaktor(
            final Held held,
            final SonderfertigkeitWert wert ) {

        float faktor = 1.0F;

        if ( held.hatVerbilligt( wert ) ) {

            faktor = 0.5F;

        } else if ( held.hat( VorNachteil.ELFISCHE_WELTSICHT )
                 && ( ! held.hatLeitsonderfertigkeit( wert ) ) ) {

            faktor = 1.5F;
        }

        return faktor;
    }
}
