/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.sonderfertigkeiten.kosten;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.Sonderfertigkeit;
import de.bernhardjung.dsaprogramm.sonderfertigkeiten.SonderfertigkeitWert;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteil;

/**
 * Berechnet die Kosten fuer die Sonderfertigkeit Kulturkunde.
 *
 * @author Bernhard Jung
 */
class KostenKulturkunde extends KostenAbstrakt {

    /** Die AP-Kosten von Kulturkunde. */
    private static final int AP_KOSTEN = 150;

    public int getApKosten(Sonderfertigkeit sonderfertigkeit) {
        return AP_KOSTEN;
    }

    public int getApKosten(final SonderfertigkeitWert wert) {
        return AP_KOSTEN;
    }

    /**
     * @return Elfsiche Weltsicht, nandusgef. Wissen, soziale
     *         Anpassungsfaehigkeit.
     */
    public Collection<Object> getAbhaengigkeiten() {

        final List< Object > abhaengigkeiten = new ArrayList< Object >();

        abhaengigkeiten.add( VorNachteil.ELFISCHE_WELTSICHT );

        abhaengigkeiten.add( Sonderfertigkeit.NANDUSGEFAELLIGES_WISSEN );

        abhaengigkeiten.add( VorNachteil.SOZIALE_ANPASSUNGSFAEHIGKEIT );

        return abhaengigkeiten;
    }

    public int getApKosten(
            final Held held,
            final SonderfertigkeitWert wert ) {

        if ( held.hatVorgegeben( wert ) ) {
            // Durch Rasse, Kultur oder Profession vorgegebene
            // Sonderfertigkeiten kosten nichts.
            return 0;
        }

        float faktor = 1.0F;

        if ( held.hatVerbilligt( wert ) ) {

            faktor = 0.5F;

        } else if ( held.hat( VorNachteil.ELFISCHE_WELTSICHT )
                && ( ! held.hatLeitsonderfertigkeit( wert ) ) ) {

           faktor = 1.5F;
        }

        if ( ( ! held.hatVerbilligt( wert ) )
          && ( held.hat( VorNachteil.SOZIALE_ANPASSUNGSFAEHIGKEIT )
            || held.hat( Sonderfertigkeit.NANDUSGEFAELLIGES_WISSEN ) ) ) {

            faktor *= 0.5F;

        }

        return Math.round( faktor * AP_KOSTEN );
    }

}
