/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.talente.stufen;

import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;
import de.bernhardjung.dsaprogramm.talente.Sprache;
import de.bernhardjung.dsaprogramm.talente.Talent;

/**
 * Bestimmt die Hoechststufe einer Sprache entsprechend ihrer Komplexitaet.
 */
class StufenSprache extends StufenTalent {

    @Override
    public Collection<Object> getAbhaengigkeiten() {

        final Collection< Object > abhaengigkeiten = super.getAbhaengigkeiten();

        abhaengigkeiten.add( Eigenschaft.KL );

        return abhaengigkeiten;
    }

    /**
     * Vergibt einen Bonus von KL-2 auf die Muttersprache und von KL-4 auf die
     * Zweitsprache.
     */
    @Override
    public int getModifikation( final Held held, final Talent talent ) {

        int modifikation = super.getModifikation( held, talent );

        final Sprache sprache = (Sprache) talent;

        if ( held.wirdErschaffen() ) {

            if ( held.hatMuttersprache( sprache ) ) {

                modifikation += held.getStufe( Eigenschaft.KL ) - 2;

            } else if ( held.hatZweitsprache( sprache ) ) {

                modifikation += held.getStufe( Eigenschaft.KL ) - 4;

            }
        }

        return modifikation;
    }

    /** {@inheritDoc} */
    @Override
    public int getHoechststufe( final Held held, final Talent talent ) {

        // Die Hoechststufe durch die Eigenschaften
        final int hoechststufe = super.getHoechststufe( held, talent );

        final int komplexitaet = ((Sprache) talent).getKomplexitaet();

        return Math.min( hoechststufe, komplexitaet );
    }
}
