/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.vornachteile.anpasser;

import de.bernhardjung.dsaprogramm.basis.Augenfarbe;
import de.bernhardjung.dsaprogramm.basis.Haarfarbe;
import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.bausteine.Rasse;
import de.bernhardjung.dsaprogramm.vornachteile.VorNachteilWert;

/**
 * Diese Klasse aendert die Haar- und Augenfarbe des Helden passend zum
 * Nachteil Albino (weiss, rot), bzw gibt ihm wieder eine normale Haar- und
 * Augenfarbe, wenn der Nachteil abgewaehlt wird.
 *
 * @author Bernhard Jung
 */
class AnpasserAlbino implements Anpasser {

    /**
     * Setzt die Augenfarbe des Helden auf rot und die Haarfarbe auf weiss.
     */
    public void eingefuegt( final Held held, final VorNachteilWert wert ) {

        held.setAugenfarbe( Augenfarbe.ROT );

        held.setHaarfarbe( Haarfarbe.WEISS );
    }

    /**
     * Aendert die Haar- und Augenfarbe des Helden auf fuer seine Rasse
     * uebliche Farben.
     */
    public void entfernt( final Held held, final VorNachteilWert wert ) {

        final Rasse rasse = held.getRasse();

        final String augenfarbe = rasse.getAugenfarben().wuerfeln();

        held.setAugenfarbe( augenfarbe );

        final String haarfarbe = rasse.getHaarfarben().wuerfeln();

        held.setHaarfarbe( haarfarbe );

    }

    /** Macht nichts. */
    public void stufeGeaendert(final Held held, final VorNachteilWert wert) {
        // kein Implementierung noetig.
    }

    public void geaendertVariante(
            final Held held,
            final Object alteVariante,
            final Object neueVariante ) {
        // nichts zu tun.
    }

}
