/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.vornachteile.stufen;

import java.util.Arrays;
import java.util.Collection;

import de.bernhardjung.dsaprogramm.basis.Held;
import de.bernhardjung.dsaprogramm.eigenschaften.Eigenschaft;

/**
 * In Ausruestungsvorteil darf man maximal SO GP inverstieren.
 * 
 * @author Bernhard Jung
 */
class StufenAusruestungsvorteil implements Stufen {

    private static final int MINDESTSTUFE = 1;
    
    private static final int SCHRITTWEITE = 1;

    /** @return eine leere Liste. */
    public Collection<Object> getAbhaengigkeiten() {
        return Arrays.asList( (Object) Eigenschaft.SO );
    }
    
    /** @return den SO des Helden */
    public final int getHoechststufe( final Held held ) {
        return held.getStufe( Eigenschaft.SO );
    }

    /** @return 1 */
    public final int getMindeststufe() {
        return MINDESTSTUFE;
    }

    /** @return 1 */
    public final int getMindeststufe( final Held held ) {
        return MINDESTSTUFE;
    }

    /** @return 1 */
    public final int getSchrittweite() {
        return SCHRITTWEITE;
    }

    /** @return {@code true} */
    public boolean hatStufen() {
        return true;
    }

}
