/*
    This file is part of the TDE Character Generator (TCG).

    TCG is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    TCG is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with TCG.  If not, see <http://www.gnu.org/licenses/>.
*/
package de.bernhardjung.dsaprogramm.zauber;

import de.bernhardjung.dsaprogramm.i18n.I18n;

/**
 * Die Repraesentation eines Zaubers, gib an ob es sich um einen Magier- oder
 * Elfenzauber handelt.
 *
 * @author Bernhard Jung
 *
 */
public enum Repraesentation {

    MAGIER( "Repraesentation.Magier" ),
    ELFEN(  "Repraesentation.Elfen" );

    /**
     * Liefert eine Repraesentation mit einer bestimmten Id.
     *
     * @param id der gesuchten Repraesentation.
     * @return die Repraesentation mit der uebergebenen Id.
     * @throws IllegalArgumentException wenn es keinen Repraesentation mit der
     *         uebergebenen Id gibt.
     */
    public static final Repraesentation getRepraesentation( final String id ) {

        if ( id.equals( "Repraesentation.Magier" ) ) {

            return MAGIER;

        } else if ( id.equals( "Repraesentation.Elfen" ) ) {

            return ELFEN;

        } else {

            throw new IllegalArgumentException( "Es gibt keine " +
            		"Repraesentation mit Id " + id );
        }
    }

    private final String id;

    private final String name;

    private Repraesentation( final String dieId ) {
        id = dieId;
        name = I18n.i18n( id );
    }

    /** @return die Id der Repraesentation. */
    public final String getId() {
        return id;
    }

    /** @return den Namen der Repraesentation, "Magier" oder "Elfen". */
    public final String getName() {
        return name;
    }

    /** @return den Namen der Repraesentation, "Magier" oder "Elfen". */
    @Override
    public String toString() {
        return name;
    }
}
